#   laser/tcl/acq.tcl
#     12/02/00   for Laser group adapted from other versions

set debug 0
set started 0

set frame ACQ
set frametitle "Laser Data Acquisition"
set icontitle  "Acquisition"

set bfopened 0

set sdresource hist

set selection {}
set spectrumlist {}

set chanmenu [list 50 100 200]

set scaler1spec "Scaler1"
set scaler2spec "Scaler2"
set scaler3spec "Scaler3"
set scaler4spec "Scaler4"
set singspec "Singles"
set adcspec  "ADC"
set patternspec  "Pattern"
set eventbuf "EventBuffer"

set scaler1 1
set scaler2 0
set scaler3 0
set scaler4 0

set autoScaler1 0
set autoScaler2 0
set autoScaler3 0
set autoScaler4 0
set autoSingles 0
set autoADC 0
set autoPattern 0
set autoEbyE 0
set autoperiod [expr 5 * 60 * 1000]

# These variables must match the enumerated types in laserdrvr.c
# not going states
set HALTED 0
set PAUSED 1
# going states
set GOING 2
set RESUMING 3
# stopping states
set STOP_NOW 4
set STOP_AT_END 5
set PAUSE_NOW 6
set PAUSE_AT_END 7

set EBYE_WANTED 1
set EBYE_GOING 2
set EBYE_PAUSE 4
set EBYE_WANTED_PAUSE 5

set stopmode $STOP_AT_END

set pausemode $PAUSE_NOW

set dacstep 2
set dacmax 0
set dacmaxmv 0.0
set rampit 1

# 11-bit ADC (+ve part of 12-bits)
set adcchans 2048

# 8-bit pattern register
set patternchans 256

# 16 MB total event buffer size, each event is 4 words, each entry is 4 bytes
# + 1 word for the event counter

set eventsize 4
set ebufsize [expr 16 * 1024 * 1024 - 4]
set maxevents [expr $ebufsize / $eventsize / 4]
set evspecsize [expr $maxevents * $eventsize + 1]

set refreshperiod 5000
set firstrefresh 0
set acqafterid ""
set ebyeafterid ""

set ebyesave 0

set stopping 0
set stopwait 0

proc kill-after id {after cancel $id}

proc dirnames l {
   foreach f $l {lappend ll [first $f]}
   return $ll
}

proc specdirnames l {
   set ll {}
   foreach f $l {lappend ll [second $f]}
   return $ll
}

proc prefix-list {l p} {
   set ll {}
   foreach f $l {lappend ll $p$f}
   return $ll
}

proc delete-spectrum n {
   global sdresource EGallow
   set a $EGallow
   set EGallow 0x30004
   catch { EG "spectrum :path '/$sdresource' :name '$n' !delete" }
   set EGallow $a
   return
}

proc create-spectrum {n c} {
   global sdresource EGallow
   catch { EG "spectrum :path '/$sdresource' :range $c :name '$n' !new" }
   return
}

proc zero-spectrum n {
   global sdresource EGallow
   catch { EG "spectrum :path '/$sdresource' :name '$n' !zero" }
}

#  includes a kludge to NOT display the EventBuffer spectrum name
proc read-spectra {} {
   global sdresource EGallow dirlist eventbuf
   set m [lsort [specdirnames [EG "spectrum :path '/$sdresource' ?names"]]]
   set dirlist [lreplace $m [set i [lsearch $m $eventbuf]] $i]
}

proc paint-histlist {} {
   global dirlist spectrumlist
   :modify LIST  :empty T :strings $dirlist
   set spectrumlist {}
}
                 
proc resave {} {
   global ebyeafterid rstate ebyestate EBYE_GOING EBYE_PAUSE
   global stopping modulename
   global scaler1spec scaler2spec scaler3spec scaler4spec singspec adcspec patternspec eventbuf
   global autoScaler1 autoScaler2 autoScaler3 autoScaler4 autoSingles autoADC autoPattern autoEbyE
   global autoperiod dwell pausemode GOING PAUSE_NOW RESUMING

   if {$ebyeafterid!=""} then {catch {kill-after $ebyeafterid}}
   if {$autoEbyE} then {
      if {[expr $ebyestate & $EBYE_GOING || $ebyestate & $EBYE_PAUSE]} then {
         if {$rstate==$GOING} then {
            set resume 1
            set-footer "Pausing Data Acquisition to Save EbyE Data"
            set stopping 1
            erswrite $modulename.Run $PAUSE_NOW
            after [expr $dwell + 50]
         } else {
           set resume 0
         }
         set-footer "Auto Saving EbyE Data"
         acqs-auto-write-ebye $eventbuf 1
# Auto save the EbyE data every five minutes
         set ebyeafterid [after $autoperiod resave]
         if {$resume} then {
            set stopping 0
            erswrite $modulename.Run $RESUMING
            set-footer "Data Acquisition Resuming"
            after [expr $dwell * 2] do-redisplay
         }
      }
   }
}

proc refresh {} {
	global modulename dwell stopwait HALTED
	global acqafterid ebyeafterid firstrefresh refreshperiod rstate
	global ebyestate ebye EBYE_GOING ebyesave stopping
	global scaler1spec scaler2spec scaler3spec scaler4spec singspec adcspec patternspec eventbuf
    global autoScaler1 autoScaler2 autoScaler3 autoScaler4 autoSingles autoADC autoPattern autoEbyE


	clear-footer
	set rstate 0
	do-redisplay
	if {$acqafterid!=""} then {catch {kill-after $acqafterid}}
	switch -- $rstate  {
                0  {
			set-footer "Data Acquisition Complete"
			if {$ebyeafterid!=""} then {catch {kill-after $ebyeafterid}}
			click-save 0
			:modify EBYE :inactive F

#			if {!$stopping} then {midas-warning "Data Acquisition stopped unexpectedly\nIs the EbyE event buffer full?"}
			inform sd stop-refresh
			inform sd stop-refresh
			set stopping 0
		    }

                1  {
			set-footer "Data Acquisition Paused"
			set stopping 0
		   }

                2  {
			if {$firstrefresh} then {
			   if {$firstrefresh==1} then {
			      incr firstrefresh
			   } else {
			      maybe-show 2
                              set firstrefresh 0
			   }
			}
			set acqafterid [after $refreshperiod refresh]
			set ebyesave [expr $ebyestate & $EBYE_GOING]
			set-footer "Data Acquisition in Progress"
		   }

                3   {
			set acqafterid [after $refreshperiod refresh]
			set-footer "Data Acquisition Resuming" 
		    }

                4   {
# a fix to kill it off if it sticks in stopping state
			if {$stopwait > [expr $dwell/$refreshperiod + 2]} then {
			   erswrite $modulename.Run $HALTED
			}
			incr stopwait

			set acqafterid [after $refreshperiod refresh]
			set-footer "Data Acquisition Stopping" 
		    }

                5   {
			set acqafterid [after $refreshperiod refresh]
			set-footer "Data Acquisition Stopping" 
		    }

                6   {
			set acqafterid [after $refreshperiod refresh]
			set-footer "Data Acquisition Pausing" 
		    }

                7   {
			set acqafterid [after $refreshperiod refresh]
			set-footer "Data Acquisition Pausing" 
		    }
	}
}

proc list-take {as is} {
   set ll {}
   foreach e $is {if {$e!=-1} then {lappend ll [lindex $as $e]}}
   return $ll
}

proc list-give {as ns} {
   set ll {}
   foreach e $ns {lappend ll [lsearch $as $e]}
   return $ll
}

proc get-spectrum-list {} {
# get the currently selected spectra
	global spectrumlist 
	return $spectrumlist
}

proc click-save t {
   global ebyesave ebyestate EBYE_GOING
   global scaler1spec scaler2spec scaler3spec scaler4spec singspec adcspec patternspec eventbuf
   global autoScaler1 autoScaler2 autoScaler3 autoScaler4 autoSingles autoADC autoPattern autoEbyE

   if {$autoScaler1} then {acqs-temp-save $scaler1spec}
   if {$autoScaler2} then {acqs-temp-save $scaler2spec}
   if {$autoScaler3} then {acqs-temp-save $scaler3spec}
   if {$autoScaler4} then {acqs-temp-save $scaler4spec}
   if {$ebyesave} then {
#      set ebyesave [expr $ebyestate & $EBYE_GOING]
      if {$autoSingles} then {acqs-temp-save $singspec}
      if {$autoADC} then {acqs-temp-save $adcspec}
      if {$autoPattern} then {acqs-temp-save $patternspec}
      if {$autoEbyE} then {acqs-auto-write-ebye $eventbuf $t}
   }
}

proc click-zero {} {
   global sdresource EGallow
   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "zeroing /$sdresource/$spectrum..."
      zero-spectrum $spectrum
      incr c
   }
   set-footer "total of $c spectra successfully zeroed"
}

proc click-show s {
   global frame sdresource EGallow
   set list [get-spectrum-list]
   if {$list!={}} then {
      inform sd "show [prefix-list $list /$sdresource/]"
      if {$s} then {
         open-frame $frame
         if {$s==1} then {
            set v [send sd inquire viewmode]
            send sd click-viewmode all
            send sd click-scale D
            send sd click-scale D
            send sd click-scale D
            send sd click-viewmode $v
         }
      }
   }
}

proc maybe-show s {
   if {[is-started sd]} then {
      set r [send sd inquire refreshafter]
      send sd stop-refresh
      send sd after cancel $r
#      click-show $s
      click-select-all
      click-show 0
      if {$r == ""} then {
         send sd click-refresh 5
      } else {
         send sd start-refresh
      }
   }
   return
}

proc click-enable-scalers v {
   global scaler1 scaler2 scaler3 scaler4
   set scaler1 [expr ($v &  1) ? 1 : 0]
   set-scaler 1 $scaler1
   set scaler2 [expr ($v &  2) ? 1 : 0]
   set-scaler 2 $scaler2
   set scaler3 [expr ($v &  4) ? 1 : 0]
   set-scaler 3 $scaler3
   set scaler4 [expr ($v &  8) ? 1 : 0]
   set-scaler 4 $scaler4
}

proc click-auto-specs v {
   global autoScaler1 autoScaler2 autoScaler3 autoScaler4
   global autoSingles autoADC  autoPattern autoEbyE
   global ebyeafterid ebyestate EBYE_GOING

   set autoScaler1 [expr $v &  1]
   set autoScaler2 [expr $v &  2]
   set autoScaler3 [expr $v &  4]
   set autoScaler4 [expr $v &  8]
   set autoADC     [expr $v & 16]
   set autoSingles [expr $v & 32]
   set autoPattern [expr $v & 64]
   set autoEbyE    [expr $v & 128]
   if {[expr $ebyestate & $EBYE_GOING]} then {
      if {$ebyeafterid!=""} then {catch {kill-after $ebyeafterid}}
      if {$autoEbyE} then {resave}
   }
}

proc show_main_menu {} {
   global env
   global bfopened
#    see if security option enabled and check permissions if it is
   if {$env(MIDASSECURITY) != "disabled"} then {
      if {![inform security validate "bf-menu,c"]} then {
#    user does not have permission to open bf menu
         set msg "Access Denied\n\nA password is required to access the main menu and you have not supplied it"
         set z [catch {midas_dialog [self frame-name].warningDialog "Attempted Security Violation" \
           "$msg" "warning" 0 "OK" "supply password"} m]
         if {$z != 0} then {
            puts stderr "midas-warning: [self frame-name].warningDialog: $m"
            catch {destroy [self frame-name].warningDialog}
         } else {
            if {$m == 1} then {inform security start}
         }
         return "Access Denied"
      }
   }
   set bfopened 1
   send bf open-frame
}

#    Check if task is started
proc is-started {task} {
    inform $task inquire started
}

proc starttask {task} {
#    Check if it is necessary to pre-start resource interface task

    set started [inform $task inquire started]
    if {$started == 0} then {send $task make-frame}
}

proc click-redisplay-spectra {} {
   global dirlist spectrumlist selection
   read-spectra
   paint-histlist
   set spectrumlist [list-take $dirlist $selection]
   list-repaint
}
  
proc click-deselect-all {} {
   global spectrumlist selection
   click-redisplay-spectra
   :modify LIST :select
   set spectrumlist {}
   set selection {}
}

proc click-select-all {} {
   global dirlist spectrumlist selection
   click-redisplay-spectra
   :modify LIST :select "0 end"
   set spectrumlist $dirlist
   set selection [[:path list].list curselection]
}

proc click-redisplay {} {refresh}

proc do-redisplay {} {
   global rstate curchan HALTED
   set-footer "Updating display"
   if {[configure-registers] == 1} then {
      parameters-restore
      write-channel-parameters
   }
   click-redisplay-spectra
   update-widget RUN-NUM [run-num-read]
   update-run-status
   if {$rstate==$HALTED} then {:modify EBYE :inactive F}
   update-channel-parameters
}

proc click-list {s} {
   global spectrumlist selection dirlist
   set selection $s
   set spectrumlist {}
   foreach i $s {lappend spectrumlist [lindex $dirlist $i]}
}

proc list-repaint {} {
   global selection
   :modify LIST :select -1
   foreach i $selection {if {$i!=-1} then {:modify LIST :select $i}}
}

proc label-online-spectra {} {
   global sdresource EGallow scaler1spec scaler2spec scaler3spec scaler4spec singspec adcspec patternspec eventbuf
   global chans dacstep dacmax dacmaxmv dwell settle
   global ebyestate ebye

   set run [run-num-read]

   set infolist "{Channels: \"$chans\"} {DAC Step: \"$dacstep\"} {Maximum mV: \"$dacmaxmv\"} {Dwell: \"$dwell\"} {Settle: \"$settle\"}"

   set title "Scaler 1 - Run $run"
   EG "spectrum :path '/$sdresource' :name '$scaler1spec' :title '$title'"
   EG "spectrum :path '/$sdresource' :name '$scaler1spec' :infonum 3 :info '$infolist'"

   set title "Scaler 2 - Run $run"
   EG "spectrum :path '/$sdresource' :name '$scaler2spec' :title '$title'"
   EG "spectrum :path '/$sdresource' :name '$scaler2spec' :infonum 3 :info '$infolist'"

   set title "Scaler 3 - Run $run"
   EG "spectrum :path '/$sdresource' :name '$scaler3spec' :title '$title'"
   EG "spectrum :path '/$sdresource' :name '$scaler3spec' :infonum 3 :info '$infolist'"

   set title "Scaler 4 - Run $run"
   EG "spectrum :path '/$sdresource' :name '$scaler4spec' :title '$title'"
   EG "spectrum :path '/$sdresource' :name '$scaler4spec' :infonum 3 :info '$infolist'"

   if {$ebye} then {
      set title "Singles - Run $run"
      EG "spectrum :path '/$sdresource' :name '$singspec' :title '$title'"
      EG "spectrum :path '/$sdresource' :name '$singspec' :infonum 3 :info '$infolist'"

      set title "ADC - Run $run"
      EG "spectrum :path '/$sdresource' :name '$adcspec' :title '$title'"
      EG "spectrum :path '/$sdresource' :name '$adcspec' :infonum 3 :info '$infolist'"

      set title "Pattern - Run $run"
      EG "spectrum :path '/$sdresource' :name '$patternspec' :title '$title'"
      EG "spectrum :path '/$sdresource' :name '$patternspec' :infonum 3 :info '$infolist'"

      set title "Event Buffer - Run $run"
      EG "spectrum :path '/$sdresource' :name '$eventbuf' :title '$title'"
      EG "spectrum :path '/$sdresource' :name '$eventbuf' :infonum 3 :info '$infolist'"

   }
}

proc check-DA-started s {
   global modulename firstrefresh
   global acqafterid ebyeafterid refreshperiod autoperiod
   global rstate ebyestate ebye GOING STOP_NOW EBYE_GOING

   read-status
   if {$rstate!=$GOING} then {
      set-footer "Data Acquisition failed to start"
      :modify EBYE :inactive F
      update-run-status
      midas-warning "Data Acquisition failed to start"
      return
   }
   if {$ebye} then {
      if {![expr $ebyestate & $EBYE_GOING]} then {
         erswrite $modulename.Run $STOP_NOW
         set-footer "Data Acquisition Stopped"
         :modify EBYE :inactive F
         update-run-status
         midas-warning "Event-by-Event Acquisition failed to start"
         return
      } else {
         set ebyeafterid [after $autoperiod resave]
      }
   }
#   if {$s} then {maybe-show 1}
   set firstrefresh 1
   refresh
}

proc click-gostop v {
   global modulename EGallow stopping stopwait dwell
   global acqafterid refreshperiod rstate stopmode GOING PAUSED HALTED
   global ebyestate ebye
   global chans curchan adcchans patternchans evspecsize
   global scaler1spec scaler2spec scaler3spec scaler4spec singspec adcspec patternspec eventbuf

   if {$v==0} then {
      if {$rstate==$GOING} then {
         update-run-status
         return
      }
      if {$rstate!=$HALTED} then {
         midas-warning "Can't start Data Acquisition.\nMake sure the last run is halted (not just paused)"
         update-run-status
         return
      }
      set stopping 0
      set stopwait 0
      delete-spectrum $scaler1spec
      create-spectrum $scaler1spec $chans
      zero-spectrum $scaler1spec
      delete-spectrum $scaler2spec
      create-spectrum $scaler2spec $chans
      zero-spectrum $scaler2spec
      delete-spectrum $scaler3spec
      create-spectrum $scaler3spec $chans
      zero-spectrum $scaler3spec
      delete-spectrum $scaler4spec
      create-spectrum $scaler4spec $chans
      zero-spectrum $scaler4spec
      delete-spectrum $singspec
      delete-spectrum $adcspec
      delete-spectrum $patternspec
      if {$ebye} then {
         create-spectrum $singspec $chans
         zero-spectrum $singspec
         create-spectrum $adcspec $adcchans
         zero-spectrum $adcspec
         create-spectrum $patternspec $patternchans
         zero-spectrum $patternspec
         delete-spectrum $eventbuf
         create-spectrum $eventbuf $evspecsize
         zero-spectrum $eventbuf
      }
      set run [run-num-incr-and-save]
      label-online-spectra
      erswrite $modulename.Event 0
      set ega $EGallow; set EGallow 0x2000a
      set m [erswrite $modulename.Run $GOING]
      set EGallow $ega
      if {$m == "0x2000a"} then {
         midas-warning "Couldn't start Data Acquisition.\nIs your Maximum DAC voltage too great?"
         update-run-status
         return
      }
      :modify EBYE :inactive T
      update-widget PAUSEIT 0
      update-widget CHANNEL 0
      after [expr 5 * $dwell] check-DA-started 1
      set-footer "Data Acquisition Starting"
   } else {
      set stopping 1
      if {$rstate==$PAUSED} then {set stop $HALTED;update-widget PAUSEIT 0} else {set stop $stopmode}
      erswrite $modulename.Run $stop
      update-run-status
      refresh
      after [expr ($chans - $curchan) * $dwell + 1000] refresh
      set-footer "Data Acquisition Stopping"
   }
}

proc click-pauseit v {
  if {$v==1} then {click-pause} else {click-continue}
}

proc click-pause {} {
   global modulename stopping pausemode RESUMING dwell
      set stopping 1
      erswrite $modulename.Run $pausemode
      set-footer "Data Acquisition Pausing"
      after [expr $dwell + 50] do-redisplay
}

proc click-continue {} {
   global modulename stopping pausemode RESUMING dwell
      set stopping 0
      erswrite $modulename.Run $RESUMING
      set-footer "Data Acquisition Resuming"
      after [expr $dwell * 2] check-DA-started 1
}

proc update-DAC-maxmv {} {
   global dacmaxmv
   read-parameters
   update-widget DAC-MAX $dacmaxmv
}

proc update-channel-parameters {} {
   global parameters chans pulses dacstep dacmax dacmaxmv dwell settle rampit
   read-parameters
   update-widget CHANS	$chans
   update-widget PULSES	$pulses
   update-widget DAC-STEP $dacstep
   update-widget DAC-MAX $dacmaxmv
   update-widget DWELL $dwell
   update-widget SETTLE	$settle
   update-widget DAC-RAMPIT $rampit
   update-scaler-widget
}

proc update-scaler-widget {} {
   global scaler1 scaler2 scaler3 scaler4
   set b 1
   set v 0
   for {set i 1} {$i<5} {incr i} {
      if {[set scaler$i]} then {set v [expr $v + $b]}
      set b [expr $b * 2]
   }
   update-widget ENABLE-SCALERS $v
}

proc update-run-status {} {
   global runstatus rstate curchan event STOP_NOW
   global ebyestate ebye maxevents EBYE_GOING scans
   read-status
   if {$rstate>1} then {set v 0} else {set v 1}
   update-widget GOSTOP $v
   if {$rstate>3} then {set s 1} else {set s 0}
   update-widget STOPPING $s
   update-widget CHANNEL $curchan
   update-widget EBYE $ebye
   update-widget SCANS $scans
   if {$ebye} then {update-widget EVENT $event} else {update-widget EVENT "N/A"}
   if {[expr $ebyestate & $EBYE_GOING]} then {
      if {!$v && $event >= $maxevents} then {
         midas-warning "The EbyE event buffer is full\nData Acquisition will be stopped"
         after 1000 click-acq 1
      }
   }
}

proc click-ebye v {
   global modulename ebye
   global singspec adcspec patternspec chans adcchans patternchans

   if {!$v} then {
      delete-spectrum $singspec
      delete-spectrum $adcspec
      delete-spectrum $patternspec
      update-widget EVENT "N/A"
   } else {
      create-spectrum $singspec $chans
      create-spectrum $adcspec $adcchans
      create-spectrum $patternspec $patternchans
   }
   click-redisplay-spectra
   set ebye $v
   erswrite $modulename.EbyE $ebye
}

proc click-chanmenu v {
   global chans
   set z [scan $v "%d" n]
   if {$z==1} then {click-chans $n}
   update-widget CHANS   $chans
}

proc click-chans c {
   global modulename chans
   set chans $c
   erswrite $modulename.Chans $chans
   update-DAC-maxmv
   inform calib click-redisplay
}

proc click-pulses p {
   global modulename pulses
   set pulses $p
   erswrite $modulename.Pulses $pulses
}

proc click-dac-step v {
   global modulename dacstep
   set dacstep $v
   erswrite $modulename.DACstep $dacstep
   update-DAC-maxmv
   inform calib read-parameters
   inform calib set-test-chan
   inform calib click-redisplay
}

proc click-dwell {d} {
   global modulename dwell
   set dwell $d
   erswrite $modulename.Dwell $dwell
}

proc click-dac-rampit {r} {
   global modulename rampit
   set rampit $r
   erswrite $modulename.Rampit $rampit
}

proc click-settle {s} {
   global modulename settle
   set settle [expr ($s / 4) * 4]
   update-widget SETTLE	$settle
   erswrite $modulename.Settle $settle
}

proc click-last {} {
   parameters-restore
   write-channel-parameters
   do-redisplay
}

proc double-click arg {
   global debug
   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   click $arg
   click-show 0
}

proc click arg {
	global debug
	if {$debug} then {puts stderr "click with argument \{$arg\}"}
	set f [lindex $arg 0]
	set c [lindex $arg 1]
	set w [lindex $arg 2]
	set v [lindex $arg 3]
	set i [lindex $arg 4]
	if {$c==":quit"} then {return [click-quit]}

	switch -glob $w  {
      ACQS-*      {click-acqs $arg}
      GOSTOP  {click-gostop $v}
      PAUSEIT  {click-pauseit $v}
      EBYE    {click-ebye $v}
      LIST    {click-list $i}
      SELECT-ALL    {click-select-all}
      DESELECT-ALL  {click-deselect-all}
      SHOW     {click-show 0}
      ENABLE-SCALERS    {click-enable-scalers $v}
      AUTO-SPECS    {click-auto-specs $v}
      SAVE     {click-save 1}
      CHANMENU    {click-chanmenu $v}
      CHANS    {click-chans $v}
      PULSES    {click-pulses $v}
      DAC-RAMPIT  {click-dac-rampit $v}
      DAC-STEP  {click-dac-step $v}
      DWELL     {click-dwell $v}
      SETTLE    {click-settle $v}
      LAST      {click-last}
      CALIB     {click-calib}
      EXTRAS    {click-extras $v}

		REDISPLAY {click-redisplay}
		HELP {inform mb goto [transparent "System Control"]}

		default {puts stdout $arg}
	}
}

proc start args {
   global started
   if {$started == 0} then {
      make-frame
   } else {open-frame}
   set started 1
   do-redisplay
}

proc click-extras v {
   switch -glob $v  {
      "Show Main Menu"  {show_main_menu}
      "LaserScan Sort" {inform lsrscan start}
   }
}

proc click-quit args {
   global bfopened
   close-frame
   if {!$bfopened} then {send bf click-quit}
}

proc finish args {destroy-frame}

proc frame-quit win  {
   global bfopened
   catch {wm iconify [winfo toplevel $win]}
   wm withdraw [winfo toplevel $win]
   if {!$bfopened} then {send bf click-quit}
}

proc make-frame {} {
  global env rampit chanmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_crate" \
      :icon-name "[self icon-title]" :resize F

#    :enable [self frame-name] Enter
	  
   :panel PC :layout L :padx 10 :pady 10
      :layout H; :row-gap 15; :col-gap 15
      :choice GOSTOP :label "Data Acquisition:" :strings [list GO STOP] :value 1
      :choice PAUSEIT :exclusive F :label "" :strings "PAUSE"
      :checkbox STOPPING :label "Stopping:" :value 0

      :next-row T; :col-gap 10
      :text CHANNEL :label "Current Channel:" :w 5 :deaf T
      :col-gap 5
      :text SCANS :label "Completed Scans:" :w 5 :deaf T
      :next-row T; :col-gap 5
      :message ME :label "Event-by-Event Data" :bold T
      :checkbox EBYE :label "Collect EbyE:"
      :text EVENT :label "Event Counter:" :w 7 :deaf T

   :panel PL :layout L :padx 10 :pady 5
   :layout H; :col-gap 10; :row-gap 10

   :list LIST :rows 6 :cols 30 :exclusive F :double-click T

   :panel PS :padx 10 :pady 5 :layout B :after FOOTER
      :layout H; :col-gap 10; :row-gap 10

      :message MS :label "VME Histograms" :bold T
      :col-gap 10
      :button SELECT-ALL  :label "Select All"
      :button DESELECT-ALL  :label "Deselect All"
      :col-gap 25
      :button SHOW  :label "Show Selected Spectra" :command "click-show 0"
      :next-row T
      :choice ENABLE-SCALERS :label "Enable:" :exclusive F :strings [list Scaler1 Scaler2 Scaler3 Scaler4] :value 1 :valuex 6c
      :next-row T
      :choice AUTO-SPECS :label "Save:" :exclusive F :strings [list Scaler1 Scaler2 Scaler3 Scaler4 ADC Singles Pattern "EbyE Data"] :value 0
      :button SAVE  :label "Save Now" :command "click-save 1"

   :panel PA :padx 10 :pady 10 :layout B :after FOOTER
   :layout H; :col-gap 10; :row-gap 10

      :message M1 :label "Run Parameters" :bold T
      :text RUN-NUM :label "Run Number:" :w 10 :deaf T
      :col-gap 40
      :menubutton CHANMENU :label "Channels:" :strings $chanmenu
      :col-gap 5
      :number CHANS :label "" :min 2 :max 2048 :w 5
      :next-row T; :col-gap 10

      :number PULSES :label "Start Pulses per Channel:" :min 1 :max 100 :w 4
      :number DAC-STEP :label "DAC Steps per Channel:" :min 1 :max 2048 :w 5
      :text DAC-MAX :w 8 :deaf T :label "Maximum DAC Voltage (mV):"
      :next-row T
      :number DWELL :label "Dwell per Channel (ms):" :min 1 :max 60000 :w 5
      :number SETTLE :label "DAC Settling Time (us):" :min 8 :max 1000000 :inc 4 :w 7 :valuex 20c
      :checkbox DAC-RAMPIT :label "Ramp DAC:" :value $rampit

   :panel PM :padx 10 :pady 10 :layout B :after FOOTER
   :layout H; :col-gap 5; :row-gap 10
      :button REDISPLAY    :label Redisplay
      :button LAST    :label "Last Run Parameters"
      :button CALIB   :label "DAC Calibration..." :command "inform calib start"
      :button VIEW    :label "Spectrum Viewer..." :command "inform sb start"
      :menubutton EXTRAS :label "Extras:" :strings [list "Show Main Menu" "LaserScan Sort"]
      :col-gap 5
      :button HELP         :label Help...

   :show T
}

