#   laser/tcl/common.laser.tcl
#     12/02/00   for Laser group adopted from other versions

set resource acq

set modulename LASER

set VIPC616BaseAddress 0x6000
set TIP850slot 3

set chans 0
set dacstep 0
set dwell 0
set settle 0

proc eleventh p {lindex $p 10}
proc twelfth p {lindex $p 11}

proc update-widget {widget value} {
    :modify $widget :value $value
}

proc update-widget-label {widget label} {
    :modify $widget :label $label
}

proc ersdefine {register class} {
   global resource EGallow
   EG "register :name '/$resource/$register' :class '$class' !new"
}

proc ersundef {register} {
   global resource EGallow
   EG "register :name '/$resource/$register' !delete"
}

proc ersinquire {register} {
   global resource EGallow
   EG "register :pattern '/$resource/$register' ?names"
#   set EGallow 0x10008
#   catch {EG "register :pattern '/$resource/$register' ?names"} m
#   if {$m == $EGallow} then {return}
#   return $m
}

proc erswta {register args} {
   global resource EGallow
   EG "register  :name '/$resource/$register' :object '$args'"
}

proc ersread {register} {
   global resource EGallow
#set EGallow 0x10008
   EG "register :name '/$resource/$register' ?value"
}

proc erswrite {register args} {
   global resource EGallow
#set EGallow 0x10008
   EG "register :name '/$resource/$register' :value '$args'"
}

proc define-register {class name attributes} {
   global modulename EGreport
   set register $modulename.$name
   if {$EGreport!=-1} then {set-footer "defining register $register"}
   ersdefine $register $class
   if {$attributes != ""} then {erswta $register $attributes}
   clear-footer
}

proc delete-register {names} {
   ersundef $names
}

proc define-module {} {
   global resource modulename EGreport
   global VIPC616BaseAddress TIP850slot

   set egr $EGreport
   set EGreport -1

   if {$EGreport!=-1} then {set-footer "deleting all LASER registers"}
   EG "register :pattern '/$resource/$modulename.*' !delete"

   define-register laserioctl Run	 "-d laser -p rw -r SS_RUN_STATE"
   define-register laserioctl EbyE	 "-d laser -p rw -r SS_EBYE"
   define-register laserioctl Settle	 "-d laser -p rw -r SS_SETTLE_TIME"
   define-register laserioctl Dwell	 "-d laser -p rw -r SS_DWELL_TIME"
   define-register laserioctl Chans	 "-d laser -p rw -r SS_NUM_CHANS"
   define-register laserioctl DACstep	 "-d laser -p rw -r SS_DAC_STEP"
   define-register laserioctl Rampit	 "-d laser -p rw -r SS_RAMP_DAC"
   define-register laserioctl Settings.New "-d laser -p ro -r SS_NEW_SETTINGS"
   define-register laserioctl Settings	 "-d laser -p ro -r SS_ACTIVE_SETTINGS"
   define-register laserioctl Status	 "-d laser -p ro -r SS_STATUS"
   define-register laserioctl Channel	 "-d laser -p rw -r SS_CHANNEL"
   define-register laserioctl Event	 "-d laser -p rw -r SS_EVENT_COUNTER"
   define-register laserioctl Scans	 "-d laser -p ro -r SS_SCANS"
   define-register TIP850dac  DAC	 "-a $VIPC616BaseAddress -s $TIP850slot -n 0 -q m -u V -d 3 p rw"

   define-register TIP850dac DAC1	 "-a $VIPC616BaseAddress -s $TIP850slot -n1 -qm -uV -d3"
   define-register TIP850adc ADC	 "-a $VIPC616BaseAddress -s $TIP850slot -n0 -qm -uV -d3 -fx"

   set EGreport $egr
}

proc define-configuration  {server name} {
   global resource modulename

   set resource $server
   set modulename $name
}

proc configure-registers {} {
   global modulename EGreport
   set r 0
   set egr $EGreport
   set EGreport -1
   if {[ersinquire $modulename.Status] == ""} then {define-module; set r 1}
   set EGreport $egr
   return $r
}

proc read-parameters {} {
   global modulename
   global parameters chans dacstep dacmax dacmaxmv dwell settle rampit

   set parameters [ersread $modulename.Settings.New]
   set chans	[first $parameters]
   set dacstep	[second $parameters]
   set dacmax	[third $parameters]
   set dacmaxmv	[fourth $parameters]
   set dwell	[fifth $parameters]
   set settle	[sixth $parameters]
   set rampit	[ersread $modulename.Rampit]

#   set chans	25
#   set dacmax	1000
#   set dacmaxmv	3000
#   set dacstep	26
#   set dwell	2500
#   set settle	5000
}

proc read-status {} {
   global modulename runstatus rstate curchan
   global event ebyestate ebye EBYE_WANTED scans

   set scans [ersread $modulename.Scans]

   set runstatus [ersread $modulename.Status]
   set rstate     [first $runstatus]
   set curchan   [second $runstatus]
#   set mv        [third $runstatus]
#   set DAC       [fourth $runstatus]
   set ebyestate [fifth $runstatus]
   set event     [sixth $runstatus]

   set ebye [expr $ebyestate & $EBYE_WANTED]

#   set rstate     1
#   set curchan   20
#   set ebye      1
#   set ebyestate 0
#   set event     45
}

proc write-channel-parameters {} {
   global modulename chans dacstep dwell settle rampit
   erswrite $modulename.Chans $chans
   erswrite $modulename.DACstep $dacstep
   erswrite $modulename.Dwell $dwell
   erswrite $modulename.Settle $settle
   erswrite $modulename.Rampit $rampit
}


# #############################################

proc test-ADC {} {
   global ADCmV rampup

   set ADCmV 0
   set rampup 1
   next-adc   
}

proc next-adc {} {
   global modulename ADCmV rampup adcafterid

   erswrite  $modulename.DAC1 [set ADCmV]mV

   set rand  [expr 100 + int(rand() * 80 ) - 40]

   if {$rampup} then {
      set ADCmV [expr $ADCmV + $rand]
      if {$ADCmV > 10000} then {set rampup 0}
   }
   if {!$rampup} then {
      set ADCmV [expr $ADCmV - $rand]
      if {$ADCmV < 0} then {set rampup 1}
   }
   if {$rampup} then {
      set ADCmV [expr $ADCmV + $rand]
   }

   set adcafterid [after 5000 next-adc]
}


