# lsrscan.tcl
# a very rough and ready wrapper for the lsrscan sort program
# 15 July 2000 Simon Letts

set debug 0
set started 0

set frame LSR
set frametitle "LaserScan Sorting"
set icontitle  "LaserScan"

set directory "$env(HOME)/on-line/specs"
set selection {}

set scandir $env(HOME)/scan

set scanprog  mlsrscan
set scantemp  mlaser.par.template
set scanparm  mlaser.par

set docompile 1

set EBYE EbyEData

set ebyefile $env(HOME)/on-line/specs/run206.EbyEData.tmp
set bskip 0
set fskip 0
set bscan -1
set fscan 1
set statfile scan_statistics.dat
set calib [list 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0]
set hopt 2
set comp [list 1 1 1 1 1 1]
set vc 0.0


proc start args {
   global started scanparm
   if {$started == 0} then {
      make-frame
   } else {open-frame}
   set started 1
   parm-file-read $scanparm
   update-widgets
   click-redisplay
}

proc mgcompile_hist f {
  global env scandir
  set lib  [file join $scandir lib $f].o
  exec gcc -o $f -lm $lib histogram_options_lsr.c -R/MIDAS/lib_SunOS -L/MIDAS/lib_SunOS -leg
}


proc click-sort {} {
   global docompile scandir scanprog scanparm scantemp
   cd $scandir
   if {$docompile} then {
      set-footer "Compiling sort"
      mgcompile_hist $scanprog
   }
   set-footer "Starting sort. This make take some time"
   set z [catch {exec $scanprog $scanparm > $scanprog.log} m]
   set fullpath [file join $scandir $scanprog]
   if {$z} then {
     midas-warning "There was an error running $fullpath.\n\nLook at $fullpath.log."
   } else {
     midas-report "$scanprog completed successfully."
   }
   clear-footer
}

#
# read parameters
#
proc parm-file-read pf {
	global env scandir
	global ebyefile bskip fskip bscan fscan statfile calib hopt comp vc

	set fn [file join  $scandir $pf]
	set z [catch {open $fn r} f]
	if {$z!=0} then {return [parm-file-write $pf]}

	set ebyefile [gets $f]
	set xbskip [gets $f]
	set xfskip [gets $f]
	set xbscan [gets $f]
	set xfscan [gets $f]
	set xstatfile [gets $f]
	set xcalib [gets $f]
	set hopt [gets $f]
	set comp [gets $f]
	set xvc [gets $f]

	close $f
}

#
# save all parameters to file
#
proc parm-file-write pf {
	global env scandir
	global ebyefile bskip fskip bscan fscan statfile calib hopt comp vc

	set fn [file join  $scandir $pf]
	set z [catch {open $fn w} f]
	puts $f $ebyefile
	puts $f $bskip
	puts $f $fskip
	puts $f $bscan
	puts $f $fscan
	puts $f $statfile
	puts $f $calib
	puts $f $hopt
	puts $f $comp
	puts $f $vc
	close $f
	return 0
}

proc click-fname v {
   global ebyefile scanparm
   set ebyefile $v
   parm-file-write $scanparm
}

proc click-hoption v {
   global hopt scanparm
   set hopt $v
   parm-file-write $scanparm
}

proc click-comp {i v} {
   global comp scanparm
   set comp [lreplace $comp $i $i $v]
   parm-file-write $scanparm
}

proc click-c0 v {
   click-comp 0 $v
}

proc click-c1 v {
   click-comp 1 $v
}

proc click-c2 v {
   click-comp 2 $v
}

proc click-c3 v {
   click-comp 3 $v
}

proc click-c4 v {
   click-comp 4 $v
}

proc click-c5 v {
   click-comp 5 $v
}

proc click-docompile v {
   global docompile
   set docompile $v
}

proc click-directorylist {v i} {
   global directorylist selection directory
   global ebyefile scanparm

   set selection $i        

   if {[llength $selection]==0} then {
      set-footer "no file selected!!!"
      return
   }
   if {[llength $selection]>1} then {
      set-footer "multiple files selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   if {[string range $target 0 0] == "d"} then {
      set-footer "can't sort a directory!!!"
      return
   }
   set file [last $target]
   if {[last $target] == "."} then {set-footer "can't sort `.'"; return}
   if {[last $target] == ".."} then {set-footer "can't sort `.'"; return}

   set ebyefile $directory/$file
   parm-file-write $scanparm
   update-widget FNAME $ebyefile
}

proc click-directory name {
   global directory
   set directory $name
   click-redisplay
}

proc xclick-directorylist {v i} {
   global selection 
   set selection $i
   
}

proc my-filter f {
   global directorylist
   set ll {}
   foreach n $directorylist {
      if {[string range $n 0 0] == "d"} then {lappend ll $n}
      if {[string first $f [last $n]] != -1} then {lappend ll $n}
   }
   set directorylist $ll
}

proc click-redisplay {} {
   global EBYE directory selection
   :modify DIRECTORY :value "$directory"
   read-directory $directory
   my-filter $EBYE
   paint-directory $directory
   set selection {}
}

proc click-up {} {
  global directory
  set directory [directory-up $directory]
  click-redisplay
}

proc click-down {} {
  global directory selection
  set directory [directory-down $directory $selection]
  click-redisplay
}

proc update-widget {w v} {
   :modify $w :value $v
}

proc update-widgets {} {
   global ebyefile bskip fskip bscan fscan statfile calib hopt comp vc

   update-widget FNAME $ebyefile
   update-widget HOPTION $hopt
   update-widget C0 [first $comp]
   update-widget C1 [second $comp]
   update-widget C2 [third $comp]
   update-widget C3 [fourth $comp]
   update-widget C4 [fifth $comp]
   update-widget C5 [sixth $comp]
}


proc make-frame {} {
  global env

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/laser_sort" \
      :icon-name "[self icon-title]" :resize F

      :panel P1 :layout T :padx 10 :pady 10
         :layout H; :row-gap 0

         :text DIRECTORY :label "Directory:" :w 60 :shifters T
         :button UP   :label Up
         :button DOWN :label Down

      :panel P2 :layout T :padx 10
         :list DIRECTORYLIST :exclusive T :rows 10 :cols 80


      :panel PA :layout H :padx 10 :pady 10
      :layout H; :col-gap 10; :row-gap 10

      :text FNAME :label "Event-by-Event File:" :w 50 :deaf T

      :next-row T
      :number HOPTION :label "Histogram Option:"
      
      :next-row T
      :message M1 :label "Sort Constants" :bold T
      :number C0 :label "0:" :w 3
      :number C1 :label "1:" :w 3
      :number C2 :label "2:" :w 3
      :number C3 :label "3:" :w 3
      :number C4 :label "4:" :w 3
      :number C5 :label "5:" :w 3

      :next-row T;:col-gap 40
      :button SORT :label "Execute Sort" :colour green :command "click-sort"
      :checkbox DOCOMPILE :label "Recompile program before executing sort?" :value 1

    :show T
}
